
// Upcasting and late binding
//
class Note {
  private String noteName;
  private Note(String noteName) {
    this.noteName = noteName;
  }
  public String toString() { return noteName; }
  public static final Note
    MIDDLE_C = new Note("Middle C"),
    C_SHARP  = new Note("C Sharp"),
    B_FLAT   = new Note("B Flat");
    // Etc.
} ///:~
abstract class Instrument {
  public abstract  void play( Note n );
  static void tune( Instrument i ) {
    // ...
    i.play( Note.C_SHARP );   // late binding
  }
}
class Wind extends Instrument {
	@Override  
	public void play(Note n) {
	    System.out.println("Wind.play() " + n);
	}
}
class Stringed extends Instrument {
	@Override  
	public void play(Note n) {
    System.out.println("Stringed.play() " + n);
  }
}

class Brass extends Wind {
	@Override  
	public void play(Note n) {
    System.out.println("Brass.play() " + n);
  }

}
class Woodwind extends Wind { 
	public void play(Note n) {
	    System.out.println("Woodwind.play() " + n);
	 }
}

public class Music {
  public static void main(String[] args) {
    Wind flute = new Wind();
    Stringed violin = new Stringed();
    Brass frenchHorn = new Brass();
    Instrument.tune(flute);
    Instrument.tune(violin);
    Instrument.tune(frenchHorn);
    
    Instrument orchestra[]  = { flute, violin, frenchHorn, new Woodwind() };  // upcasting
    for( Instrument i : orchestra )
    	i.play(Note.B_FLAT);  // late binding
  }
} ///:~





